"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitHubApiAdapter = void 0;
const octokit_1 = require("octokit");
class GitHubApiAdapter {
    constructor() {
        this.octokit = new octokit_1.Octokit({ auth: process.env.GITHUB_ACCESS_TOKEN });
    }
    async createOrUpdateRepo(options) {
        const { owner, name } = options;
        const alreadyExists = await this.repoExists(owner, name);
        if (alreadyExists) {
            return this.updateRepo(options);
        }
        return this.createRepo(options);
    }
    async createOrUpdateLabel(options) {
        const { repoOwner, repoName, name } = options;
        const labelExists = await this.labelExists(repoOwner, repoName, name);
        if (labelExists) {
            return this.updateLabel(options);
        }
        return this.createLabel(options);
    }
    async deleteLabel(repoOwner, repoName, name) {
        const labelExists = await this.labelExists(repoOwner, repoName, name);
        if (!labelExists) {
            return;
        }
        return this.octokit.rest.issues.deleteLabel({
            owner: repoOwner,
            repo: repoName,
            name,
        });
    }
    async updateBranchProtection(options) {
        const { requiredStatusChecks, requiredStatusChecksStrict, requiredApprovingReviewCount, repoOwner, repoName, branch, enforceAdmins, requiredConversationResolution, allowDeletions, allowForcePushes, requiredLinearHistory, } = options;
        const enableRequiredStatusChecks = requiredStatusChecks && requiredStatusChecks.length > 0;
        const required_status_checks = enableRequiredStatusChecks
            ? {
                strict: requiredStatusChecksStrict || false,
                contexts: requiredStatusChecks,
            }
            :
                null;
        const required_pull_request_reviews = {
            required_approving_review_count: requiredApprovingReviewCount,
        };
        return this.octokit.rest.repos.updateBranchProtection({
            owner: repoOwner,
            repo: repoName,
            branch,
            required_status_checks,
            enforce_admins: enforceAdmins || null,
            required_pull_request_reviews,
            required_linear_history: requiredLinearHistory,
            allow_force_pushes: allowForcePushes,
            allow_deletions: allowDeletions,
            required_conversation_resolution: requiredConversationResolution,
            restrictions: null,
        });
    }
    async createCommitSignatureProtection(repoOwner, repoName, branch) {
        return this.octokit.rest.repos.createCommitSignatureProtection({
            owner: repoOwner,
            repo: repoName,
            branch: branch,
        });
    }
    async enableVulnerabilityAlerts(repoOwner, repoName) {
        return this.octokit.rest.repos.enableVulnerabilityAlerts({
            owner: repoOwner,
            repo: repoName,
        });
    }
    async repoExists(owner, name) {
        try {
            const response = await this.octokit.rest.repos.get({
                owner,
                repo: name,
            });
            return response.status === 200;
        }
        catch (error) {
            const status = error.status;
            if (status === 404) {
                return false;
            }
            throw error;
        }
    }
    async updateRepo(options) {
        const gitHubRepoOptions = this.repoOptionsToGitHubRepoOptions(options);
        return this.octokit.rest.repos.update(gitHubRepoOptions);
    }
    async createRepo(options) {
        const gitHubRepoOptions = this.repoOptionsToGitHubRepoOptions(options);
        return this.octokit.rest.repos.createForAuthenticatedUser(gitHubRepoOptions);
    }
    repoOptionsToGitHubRepoOptions(options) {
        return {
            owner: options.owner,
            repo: options.name,
            name: options.name,
            description: options.description,
            allow_auto_merge: options.allowAutoMerge,
            allow_merge_commit: options.allowMergeCommit,
            allow_rebase_merge: options.allowRebaseMerge,
            allow_squash_merge: options.allowSquashMerge,
            allow_update_branch: options.allowUpdateBranch,
            delete_branch_on_merge: options.deleteBranchOnMerge,
            has_issues: options.hasIssues,
            private: options.isPrivate,
            use_squash_pr_title_as_default: options.useSquashPrTitleAsDefault,
        };
    }
    async labelExists(repoOwner, repoName, name) {
        const response = await this.octokit.rest.issues.listLabelsForRepo({
            owner: repoOwner,
            repo: repoName,
        });
        const existingLabels = response.data;
        const existingLabelNames = new Set(existingLabels.map((label) => label.name));
        return existingLabelNames.has(name);
    }
    async updateLabel(options) {
        return this.octokit.rest.issues.updateLabel(Object.assign(Object.assign({}, options), { owner: options.repoOwner, repo: options.repoName }));
    }
    async createLabel(options) {
        return this.octokit.rest.issues.createLabel(Object.assign(Object.assign({}, options), { owner: options.repoOwner, repo: options.repoName }));
    }
}
exports.GitHubApiAdapter = GitHubApiAdapter;
//# sourceMappingURL=GitHubApiAdapter.js.map